<?php
namespace nur\sery\db\mysql;

use nur\sery\cl;
use nur\sery\db\CapacitorChannel;
use nur\sery\db\CapacitorStorage;
use nur\sery\php\func;
use nur\sery\ValueException;

/**
 * Class MysqlStorage
 */
class MysqlStorage extends CapacitorStorage {
  function __construct($mysql) {
    $this->db = Mysql::with($mysql);
  }

  /** @var Mysql */
  protected $db;

  function db(): Mysql {
    return $this->db;
  }

  const PRIMARY_KEY_DEFINITION = [
    "id_" => "integer primary key auto_increment",
  ];

  protected function _create(CapacitorChannel $channel): void {
    if (!$channel->isCreated()) {
      $cols = $this->ColumnDefinitions($channel);
      $this->db->exec([
        "create table if not exists",
        "table" => $channel->getTableName(),
        "cols" => $cols,
      ]);
      $channel->setCreated();
    }
  }

  function _exists(CapacitorChannel $channel): bool {
    $db = $this->db;
    $tableName = $db->get([
      "select table_name from information_schema.tables",
      "where" => [
        "table_schema" => $db->getDbname(),
        "table_name" => $channel->getTableName(),
      ],
    ]);
    return $tableName !== null;
  }

  function _ensureExists(CapacitorChannel $channel): void {
    $this->_create($channel);
  }

  function _reset(CapacitorChannel $channel, bool $ensureExists=false): void {
    $this->db->exec([
      "drop table if exists",
      $channel->getTableName(),
    ]);
    $channel->setCreated(false);
    if ($ensureExists) $this->_ensureExists($channel);
  }

  function close(): void {
    $this->db->close();
  }
}
